{-
Stephen's Counterpoint Analysis Program V0.1 Beta
30th May 2007
http://www.maths.tcd.ie/~icecube/
icecube@maths.tcd.ie
-}

{-
Global Variables, goes from -11 to 32
-}
strongConsonances, weakConsonances, dissonances, bannedIntervals :: [Int]
strongConsonances = [-11,-4,1,8,13,20,25,32]
weakConsonances = [-8,-7,-3,-2,4,5,6,9,10,16,17,18,21,22,28,29,30]
dissonances =  [-10,-9,-6,-5,-1,0,2,3,7,11,12,14,15,19,23,24,26,27,31]
bannedIntervals = [-25..(-12)] ++ [33..47]

maxInterval :: Int
maxInterval = 13

bannedMoves :: [(Int, Int)]
bannedMoves = [(a,b) | a <- [-1,1,0], b <- bannedIntervals ]

badDissonantMoves :: [(Int,Int)]
badDissonantMoves = [(a,b) | a <- [-1,1,0], b <- dissonances ]

badConsonantMoves :: [(Int,Int)]
badConsonantMoves = [(1,a) | a <- strongConsonances ]

badMoves :: [(Int,Int)]
badMoves = badDissonantMoves ++ badConsonantMoves ++ bannedMoves

main :: IO ()
main = do
	putStrLn "Enter Melodies:"
	a <- getLine
	putStrLn ("Entered " ++ (show a))
	putStrLn "Min Length:"
	b <- getLine
	putStrLn ("Entered " ++ (show b))
	putStrLn "2-Part Counterpoints:"
	putStrLn (show (processMelodies(read a :: [[Int]]) (read b :: Int)))
	putStrLn "3-Part Counterpoints:"
	putStrLn (show (triples(read a :: [[Int]]) (read b :: Int)))
	putStrLn "Press Any Key"
	c <- getChar
	putStrLn "Byebye"

{- triples
		= (	melody 1,
			melody 2,
			melody 3.
			indentation of 1,
			indentation of 2,
			indentation of 3,
			interval 1,
			interval 2,
			 )
-}			
triples :: [[Int]]->Int->[(Int,Int,Int,Int,Int,Int,Int,Int)]
triples melodies l =[(ma,mb,md, ia,ib,id, intb, intd)|(ma,mb,ia,ib,intb,l1)<-cpset, (mc,md,ic,id,intd,l2)<-cpset, ((ma==mc)&&(ia==ic)&&(elem (mb,md,ib,id,(intd-intb+1)) (forgetLengths cpset)))]
	where
		cpset = (processMelodies melodies l)

forgetLengths :: [(Int,Int,Int,Int,Int,Int)]->[(Int,Int,Int,Int,Int)]
forgetLengths results = [(a,b,c,d,e)| (a,b,c,d,e,f)<-results]

{- processMelodies
		= (	melody 1,
			melody 2,
			indentation of m,
			indentation of n
			interval,
			polyphonic length
			 )
-}			
processMelodies :: [[Int]]->Int->[(Int,Int,Int,Int,Int,Int)]
processMelodies  melodies l = [ (a,b,c,d,e,f) | a <- int, b <- int, (c,d,e,f) <- (counterpointSets (melodies !! a) (melodies !! b) l), ((a/=b)||(c/=d)) ]
	where
		int=[0..((length melodies)-1)]

{- counterpointSets = (	indentation of m, 
			indentation of n,
			interval, 
			polyphoniclength)	
-}

counterpointSets :: [Int] -> [Int] -> Int -> [(Int, Int, Int, Int)]
counterpointSets m n l = [(i,j,k,p)| (i,j,k,p) <- (allcounterpointSets m n l), p>=l]

allcounterpointSets :: [Int] -> [Int] -> Int -> [(Int, Int, Int, Int)]
allcounterpointSets m n l = [ (i,j,k, (pl i j k)) | i <- [0..l1], j <- [0..l2], k <- [1..maxInterval] ]
	where
		l1 = length m - l
		l2 = length n - l
		pl i j k= polyphonicLength (drop i m) (drop j n) k
{-, polyphonicLength lm ln-}

{- 	This function tells you to what 	
	length the two lines are polyphonic	-}
polyphonicLength :: [Int] -> [Int] -> Int -> Int
polyphonicLength m n i = nicePolyphony (moves m n i) 0

nicePolyphony :: [(Int,Int)] -> Int -> Int
nicePolyphony m i = if ( null || badMove ) then i else (nicePolyphony (tail m) (i+1))
	where
		null = (m==[])
		badMove = elem (head m) badMoves

moves :: [Int] -> [Int] -> Int -> [(Int,Int)]
moves m n i = getMoves (take len m)  (take len n) i
	where 
		len = min (length m) (length n)
{- 
Och! There should be some way to get rid of the tail from this, but I can't think how better to do it for the moment
-}
getMoves :: [Int] -> [Int] -> Int -> [(Int,Int)]
getMoves m n i = zip (signs m n) (tail (harmonies m n i))

harmonies :: [Int] -> [Int] -> Int -> [Int]
harmonies m n i = i : [ a + b | (a,b) <- (zip (harmonies m n i) (diffs m n)) ]

diffs :: [Int] -> [Int] -> [Int]
diffs m n = [ b-a | (a,b) <- zip dm dn ]
	where
		dm = diff m
		dn = diff n

diff :: [Int] -> [Int]
diff m = 0 : [ b - a | (a,b) <- zip m (tail m) ]

signs :: [Int] -> [Int] -> [Int]
signs m n = [signum (a*b) | (a,b) <- zip dm dn ]
	where
		dm = diff m
		dn = diff n
